#include <OneWire.h>
#include <DallasTemperature.h>
#include <BME280I2C.h>
#include <Wire.h>
#include <SHT1x.h>
#include "DHT.h"
// Data wire is plugged into pin 2 on the Arduino 
#define ONE_WIRE_BUS 4
OneWire oneWire(ONE_WIRE_BUS); 
BME280I2C bme;
// Pass our oneWire reference to Dallas Temperature. 
DallasTemperature sensors(&oneWire);

//DHT11
#define DHTPIN11 2    // what pin we're connected to
#define DHTTYPE11 DHT11   // DHT 11
DHT dht11(DHTPIN11, DHTTYPE11);
 
//DHT22
#define DHTPIN22 5   // what pin we're connected to
#define DHTTYPE22 DHT22   // DHT 22
DHT dht22(DHTPIN22, DHTTYPE22);
#define ONE_WIRE_BUS 4
#define dataPin  10
#define clockPin 11
SHT1x sht1x(dataPin, clockPin);

float temp;
int tempPin = 0;

void setup() {
  Serial.begin(9600);
  Serial.println("DHTxx test!");
  dht11.begin();
  dht22.begin();
   sensors.begin();

   while(!Serial) {} // Wait

  Wire.begin();

  while(!bme.begin())
  {
    Serial.println("Could not find BME280 sensor!");
    delay(1000);
  }

  switch(bme.chipModel())
  {
     case BME280::ChipModel_BME280:
       Serial.println("Found BME280 sensor! Success.");
       break;
     case BME280::ChipModel_BMP280:
       Serial.println("Found BMP280 sensor! No Humidity available.");
       break;
     default:
       Serial.println("Found UNKNOWN sensor! Error!");
  }
}

void loop() {
  // Wait a few seconds between measurements.
  delay(2000);

  float tempDHT11 = dht11.readTemperature();
   float humDHT11 = dht11.readHumidity();
  // Check if readings have failed
  if (isnan(tempDHT11))
  {
    Serial.println("Failed to read from DHT 11 sensor!");
    return;
  }
 
  Serial.print(" Temperature (DHT11) = ");
  Serial.print(tempDHT11);
  Serial.print(" *C ");
  Serial.print(" Humidity (DHT11) = ");
  Serial.print(humDHT11);
  Serial.println(" % ");
 
 
  // Read DHT22 temperature as Celsius
  float tempDHT22 = dht22.readTemperature();
  float humDHT22 = dht22.readHumidity();
  // Check if readings have failed
  if (isnan(tempDHT22))
  {
    Serial.println("Failed to read from DHT 22 sensor!");
    return;
  }
  Serial.print(" Temperature (DHT22) =");
  Serial.print(tempDHT22);
  Serial.print(" *C ");
  Serial.print("   Humidity (DHT22) = ");
  Serial.print(humDHT22);
  Serial.println(" %");

temp = analogRead(tempPin);  // read analog volt from sensor and save to variable temp
temp = temp * 0.48828125;      // convert the analog volt to its temperature equivalent
Serial.print(" Temperature (LM35) = ");
Serial.print(temp);     // display temperature value
Serial.print("*C");
Serial.print(" Temperature (DS18B20) = "); 

Serial.print(sensors.getTempCByIndex(0));
Serial.println(" *C "); 

float temp_c;
  //float temp_f;
  float humidity;

  // Read values from the sensor
  temp_c = sht1x.readTemperatureC();
  //temp_f = sht1x.readTemperatureF();
  humidity = sht1x.readHumidity();

  // Print the values to the serial port
  Serial.print(" Temperature (SHT11) = ");
  Serial.print(temp_c);
  Serial.print(" *C");
  Serial.print("  Humidity (SHT11) = ");
  Serial.print(humidity);
  Serial.println(" %");
  printBME280Data(&Serial);
   delay(500);

Serial.println("====================================================================================");
}

void printBME280Data
(
   Stream* client
)
{
   float temp(NAN), hum(NAN), pres(NAN);
   BME280::TempUnit tempUnit(BME280::TempUnit_Celsius);
   BME280::PresUnit presUnit(BME280::PresUnit_Pa);
   bme.read(pres, temp, hum, tempUnit, presUnit);
   client->print(" Temperature (BME280) = ");
   client->print(temp);
   client->print("°"+ String(tempUnit == BME280::TempUnit_Celsius ? 'C' :'F'));
   client->print("   Humidity (BME280) = ");
   client->print(hum);
   client->println(" % ");

   delay(1000);
}
